package com.jaggaer.app;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellValue;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Iterator;

public class App {
	/**
	 * Spreadsheet file name
	 */
	private static final String FILE_NAME = System.getProperty("user.home") + "/tmp/poiFormulaEvaluation.xlsx";;



	public static void main( String[] args ) {
		//To enable log...
		//System.setProperty("org.apache.poi.util.POILogger","org.apache.poi.util.SystemOutLogger");
		//System.setProperty("poi.log.level","0");
		
		XSSFWorkbook workbook = new XSSFWorkbook();
		XSSFSheet sheet = workbook.createSheet("JavaTest");

		int rowNum = 0;
		int colNum = 0;

		Cell cell;
		FormulaEvaluator evaluator;
		System.out.println("Creating excel...");

		Row row = sheet.createRow(rowNum++);
		cell = row.createCell(colNum++);
		cell.setCellValue(0.1d);
		cell = row.createCell(colNum++);
		cell.setCellValue("+");
		cell = row.createCell(colNum++);
		cell.setCellValue(0.2d);
		cell = row.createCell(colNum++);
		cell.setCellValue("=");
		cell = row.createCell(colNum++);
		cell.setCellFormula("A1+C1");


		for (int i=28; i<30; i++) {
			row = sheet.createRow(rowNum++);
			colNum = 0;
			cell = row.createCell(colNum++);
			cell.setCellValue((double) BigDecimal.valueOf(i).divide(BigDecimal.valueOf(100)).doubleValue());
			cell = row.createCell(colNum++);
			cell.setCellValue(" -> TRUNC(number,2) -> " );
			cell = row.createCell(colNum++);
			cell.setCellFormula("TRUNC(A" + rowNum + ",2)");
		}

		try {
			File file = new File(FILE_NAME);
			if (file.exists()) {
				file.delete();
			}
			file.createNewFile();
			FileOutputStream outputStream = new FileOutputStream(FILE_NAME);
			workbook.write(outputStream);
			workbook.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		System.out.println("Excel file create succesfully...");

		System.out.println("Now read excel just created...");


		try {
			FileInputStream excelFile = new FileInputStream(new File(FILE_NAME));
			workbook = new XSSFWorkbook(excelFile);
			sheet = workbook.getSheetAt(0);
			Iterator<Row> iterator = sheet.iterator();

			evaluator = workbook.getCreationHelper().createFormulaEvaluator();
			evaluator.evaluateAll();

			while (iterator.hasNext()) {
				Row currentRow = iterator.next();
				Iterator<Cell> cellIterator = currentRow.iterator();

				while (cellIterator.hasNext()) {
					Cell currentCell = cellIterator.next();
					evaluator.setDebugEvaluationOutputForNextEval(true);
					//evaluator.evaluateFormulaCellEnum(currentCell);//POI 3.17
					evaluator.evaluate(currentCell);//POI 3.14
					if (currentCell.getCellType() == Cell.CELL_TYPE_STRING) {
						System.out.print(currentCell.getStringCellValue());
					} else if (currentCell.getCellType() == Cell.CELL_TYPE_FORMULA || currentCell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
						System.out.print(currentCell.getNumericCellValue());
					}
					System.out.print('\t');
				}
				System.out.println();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
}
